<?php

    /**
     * User: Daschmi (daschmi@daschmi.de)
     * Date: 16.11.2020
     * Time: 08:32
     */

    class wpsg_abo extends wpsg_model {

        static $_tableName = 'WPSG_TBL_ABO';
        public static $arState = [];

        const STATE_RUNNING = 1;
        const STATE_INEXTENSION = 2;
        const STATE_EXTEND = 3;

        /**
         * Lädt die Daten der Bestellung
         */
        public function load($abo_id) {

            parent::load($abo_id);

            $this->data = $this->db->fetchRow("SELECT * FROM `".WPSG_TBL_ABO."` WHERE `id` = '".intval($abo_id)."' ");
            
        }

        public function getExpirationTime() {
            
            return strtotime($this->__get('expiration'));
            
        }

        /**
         * Gibt den Wert des Status zurück
         * @return int
         */
        public function getState() {

            $expiration_time = $this->getExpirationTime();
            $time = wpsg_time();

            if ($expiration_time < $time) {

                return self::STATE_EXTEND;

            } else {

                $wpsg_mod_abo_autoexpire_informationtime = intval(
                    wpsg_ShopController::getShop()->callMod('wpsg_mod_abo', 'getSetting', ['wpsg_mod_abo_autoexpire_informationtime', $this->getProductId()])
                );
            
                if ($wpsg_mod_abo_autoexpire_informationtime > 0) {

                    $t_inform = strtotime('-'.$wpsg_mod_abo_autoexpire_informationtime.' day', $expiration_time);

                    if ($time > $t_inform) {

                        return self::STATE_INEXTENSION;

                    } else {

                        return self::STATE_RUNNING;

                    }

                } else return self::STATE_RUNNING;

            }
                        
        }
        
        /**
         * Gibt den aktuellen Status anhand der Laufzeit zurück
         */
        public function getStateLabel() {

            return self::$arState[$this->getState()];
            
        }
        
        /**
         * Gibt die Hauptbestellung zurück, mit der das Abo gestartet wurde
         * @return wpsg_order
         */
        public function getOrder() {
            
            return wpsg_order::getInstance(intval($this->data['order_id']));
            
        }
        
        public function getProductId() {
            
            $oOrderProduct = $this->getOrderProduct();
            
            if ($oOrderProduct === false) return false;
            else return $oOrderProduct->getProductId();
            
        }
        
        public function getOrderProduct() {
            
            $order_product_id = intval($this->__get('order_product_id'));
            
            if ($order_product_id <= 0) {

                $product_key = intval($this->__get('product_key'));
                $order_id = intval($this->__get('order_id'));

                if ($product_key > 0 && $order_id > 0) {

                    $order_product_id = intval($this->db->fetchOne("SELECT `id` FROM `".WPSG_TBL_ORDERPRODUCT."` WHERE `productkey` = '".wpsg_q($product_key)."' AND `o_id` = '".wpsg_q($order_id)."' "));

                    if ($order_product_id <= 0) return false;

                }

            }

            try {

                return wpsg_order_product::getInstance($order_product_id);

            } catch (\Exception $e) {

                return false;

            }

        }

        public function getNr() {

            return $this->getOrder()->getNr();

        }

        public static function getFromOrderAndOrderProduct($order_id, $order_product_id) {

            $abo_id = intval(
                wpsg_ShopController::getShop()->db->fetchOne("SELECT `id` FROM `".WPSG_TBL_ABO."` WHERE `order_id` = '".wpsg_q($order_id)."' AND `order_product_id` = '".wpsg_q($order_product_id)."' ")
            );

            if ($abo_id <= 0) {

                // Fallback für alte Abo's / Verlängerungen

                $oOrderProduct = wpsg_order_product::getInstance($order_product_id);

                if (preg_match('/^abo_\d+_\d+$/', $oOrderProduct->getProductKey())) {

                	$abo_id = intval(explode('_', $oOrderProduct->getProductKey())[1]);

                } else {

	                $abo_id = intval(
	                    wpsg_ShopController::getShop()->db->fetchOne("SELECT `id` FROM `".WPSG_TBL_ABO."` WHERE `order_id` = '".wpsg_q($order_id)."' AND `product_key` = '".wpsg_q($oOrderProduct->getProductKey())."' ")
	                );

                }

                if ($abo_id <= 0) {

                    throw new \wpsg\Exception(wpsg_translate(
                    __('Kein Abo zu dieser Bestellung (ID:#1#) und BestellProdukt (ID:#2#) gefunden.', 'wpsg'), $order_id, $order_product_id
                    ));

                }

            }

            return wpsg_abo::getInstance($abo_id);
                        
        }
        
        public static function find($arFilter = array(), $load = true) {

            list($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);

            $strLimit = "";

            if (wpsg_isSizedArray($arFilter['limit'])) $strLimit = "LIMIT ".wpsg_q($arFilter['limit'][0]).", ".wpsg_q($arFilter['limit'][1]);

            $strQuery = "
				SELECT
					A.`id`
					".$strQuerySELECT."
				FROM
					`".WPSG_TBL_ABO."` AS A
					".$strQueryJOIN."
				WHERE
					1
					".$strQueryWHERE."
				GROUP BY 
					A.`id`
				HAVING
					1
					".$strQueryHAVING."
				ORDER BY
					".$strQueryORDER."
				".$strLimit."
			";
			 
            $arAboID = wpsg_ShopController::getShop()->db->fetchAssocField($strQuery, "id", "id");

            if (wpsg_isSizedInt($arFilter['state'])) {
 
                foreach ($arAboID as $k => $abo_id) {
                    
                    $oAbo = wpsg_abo::getInstance($abo_id);
                    
                    if ($oAbo->getState() !== intval($arFilter['state'])) {
                        
                        unset($arAboID[$k]);
                        
                        if (!$load) {
                            
                            wpsg_abo::clearCache($abo_id);
                            unset($oAbo);
                            
                        }
                        
                    }
                    
                }
                
            }
            
            if ($load !== true) return $arAboID;

            $arReturn = array();

            foreach ($arAboID as $abo_id) {

                $oAbo = wpsg_abo::getInstance($abo_id);
                $arReturn[] = $oAbo;

            }

            return $arReturn;

        }

        public static function count($arFilter) {

            list($strQuerySELECT, $strQueryWHERE, $strQueryJOIN, $strQueryHAVING, $strQueryORDER) = self::getQueryParts($arFilter);

            if (wpsg_isSizedInt($arFilter['state'])) {
            
                $arFilterAll = $arFilter; unset($arFilterAll['state']);
                
                $arAbo = self::find($arFilterAll, false);

                foreach ($arAbo as $k => $abo_id) {
                    
                    $oAbo = wpsg_abo::getInstance($abo_id);

                    if ($oAbo->getState() !== intval($arFilter['state'])) unset($arAbo[$k]);
                    
                    wpsg_abo::clearCache($abo_id); unset($oAbo);
                    
                }

                return sizeof($arAbo);
                
                
            } else {
            
                $strQuery = "
                    SELECT
                        COUNT(*)
                    FROM
                        (
                            SELECT
                                DISTINCT A.`id`
                            FROM
                                `".WPSG_TBL_ABO."` AS A
                                ".$strQueryJOIN."
                            WHERE
                                1
                                ".$strQueryWHERE." 
                            HAVING
                                1
                                ".$strQueryHAVING."											
                        ) AS innerSelect
                ";
                
                return wpsg_ShopController::getShop()->db->fetchOne($strQuery);
                
            }

        } // public static function count($arFilter)
        
        public static function getQueryParts($arFilter = array()) {

            $strQuerySELECT = "";
            $strQueryWHERE = "";
            $strQueryJOIN = ""; $strQueryJOINSuffix = "";
            $strQueryHAVING = "";
            
            $bJoinOrder = false;
			$bJoinCustomer = false;
			$bJoinCustomerAddress = false;
			$bJoinOrderProduct = false;
			$bJoinProduct = false;
            
            if (wpsg_isSizedString($arFilter['s'])) {
                
                $bJoinOrder = true;

                list($strQuerySELECTOrder, $strQueryWHEREOrder, $strQueryJOINOrder, $strQueryHAVINGOrder, $strQueryORDEROrder) = wpsg_order::getQueryParts(['s' => $arFilter['s']]);
 
                $strQueryWHERE .= $strQueryWHEREOrder;
                $strQueryJOINSuffix .= $strQueryJOINOrder;
                
            }

            if (wpsg_isSizedInt($arFilter['k_id'])) {

                $bJoinOrder = true;

                list($strQuerySELECTOrder, $strQueryWHEREOrder, $strQueryJOINOrder, $strQueryHAVINGOrder, $strQueryORDEROrder) = wpsg_order::getQueryParts(['k_id' => $arFilter['k_id']]);

                $strQueryWHERE .= $strQueryWHEREOrder;
                $strQueryJOINSuffix .= $strQueryJOINOrder;

            }
            
            if (wpsg_isSizedInt($arFilter['cdate_m'])) $strQueryWHERE .= " AND MONTH(A.`cdate`) = '".wpsg_q($arFilter['cdate_m'])."' ";
            if (wpsg_isSizedInt($arFilter['cdate_y'])) $strQueryWHERE .= " AND YEAR(A.`cdate`) = '".wpsg_q($arFilter['cdate_y'])."' ";

            if (wpsg_isSizedInt($arFilter['enddate_m'])) $strQueryWHERE .= " AND MONTH(A.`expiration`) = '".wpsg_q($arFilter['enddate_m'])."' ";
            if (wpsg_isSizedInt($arFilter['enddate_y'])) $strQueryWHERE .= " AND YEAR(A.`expiration`) = '".wpsg_q($arFilter['enddate_y'])."' ";
                       
			if (($arFilter['order']??'') === 'cdate') $strQueryORDER = " A.`cdate` ";
			else if (($arFilter['order']??'') === 'customer') { 
				
				$bJoinOrder = true;
				$bJoinCustomer = true;
				$bJoinCustomerAddress = true;
				
				$strQueryORDER = " CA.`name` ";
				
			} else if (($arFilter['order']??'') === 'payment') {
				
				$bJoinOrder = true;
				
				$strQueryORDER = " O.`type_payment` ";
				
			} else if (($arFilter['order']??'') === 'shipping') {
				
				$bJoinOrder = true;
				
				$strQueryORDER = " O.`type_shipping` ";
				
			}			
			else if (($arFilter['order']??'') === 'products') {
			
				$bJoinOrderProduct = true;
				$bJoinProduct = true;
				
				$strQueryORDER = " P.`name` ";

			} else if (($arFilter['order']??'') === 'amount') {
				
				$bJoinOrder = true;
				
				$strQueryORDER = " O.`price_gesamt_brutto` ";

			} else if (($arFilter['order']??'') === 'expiration') $strQueryORDER = " A.`expiration` ";
			else if (($arFilter['order']??'') === 'state') $strQueryORDER = " A.`status` ";
			else $strQueryORDER = " A.`id` ";
			
			if (($arFilter['ascdesc']??'') === 'DESC') $strQueryORDER .= " DESC ";
			else $strQueryORDER .= " ASC ";
			
			if ($bJoinOrder) $strQueryJOIN .= " LEFT JOIN `".WPSG_TBL_ORDER."` AS O ON (O.`id` = A.`order_id`) ";
			if ($bJoinCustomer) $strQueryJOIN .= " LEFT JOIN `".WPSG_TBL_KU."` AS C ON (O.`k_id` = C.`id`) ";
			if ($bJoinCustomerAddress) $strQueryJOIN .= " LEFT JOIN `".WPSG_TBL_ADRESS."` AS CA ON (C.`adress_id` = CA.`id`)  ";
			if ($bJoinOrderProduct) $strQueryJOIN .= " LEFT JOIN `".WPSG_TBL_ORDERPRODUCT."` AS OP ON (OP.`id` = A.`order_product_id`) ";
			if ($bJoinProduct) $strQueryJOIN .= " LEFT JOIN `".WPSG_TBL_PRODUCTS."` AS P ON (P.`id` = OP.`p_id`) ";
						
            return array($strQuerySELECT, $strQueryWHERE, $strQueryJOIN.$strQueryJOINSuffix, $strQueryHAVING, $strQueryORDER);

        }
        
        public static function init() {
         
            self::$arState = [
                self::STATE_RUNNING => __('laufend', 'wpsg'),
                self::STATE_INEXTENSION => __('in Verlängerung', 'wpsg'),
                self::STATE_EXTEND => __('abgelaufen', 'wpsg')
            ];
            
        }
        
    }

    wpsg_abo::init();